#
# Profile for configuring the bare necessities to running a Jenkins master
#
# Parameters
# ----------
#
# ci_fqdn = 'ci.jenkins.io' (Default)
#   Define the fully-qualified domain name for this Jenkins master. This value
#   will be used for Jenkins' own configuration as well as Apache virtual hosts
#   and certificates
#
# letsencrypt = true (Default)
#   Enable letsencrypt configuration, for this to work the Jenkins host has to
#   be on the public internet
#
class profile::buildmaster(
  $ci_fqdn     = 'ci.jenkins.io',
  $letsencrypt = true,
  $plugins     = undef,
  $proxy_port  = 443,
  $jenkins_home= '/var/lib/jenkins',
) {
  include ::stdlib
  include ::apache
  include apache::mod::alias
  include apache::mod::proxy
  include apache::mod::headers
  include apache::mod::rewrite

  validate_string($ci_fqdn)
  validate_bool($letsencrypt)
  validate_array($plugins)

  include profile::apachemisc
  include profile::docker
  include profile::firewall

  if $letsencrypt {
    include profile::letsencrypt
  }

  $ldap_url    = hiera('ldap_url')
  $ldap_dn     = hiera('ldap_dn')
  $ldap_admin_dn = hiera('ldap_admin_dn')
  $ldap_admin_password = hiera('ldap_admin_password')

  $ssh_dir = "${jenkins_home}/.ssh"
  $ssh_cli_key = 'jenkins-cli-key'

  $script_dir = '/usr/share/jenkins'
  $cli_script = "${script_dir}/idempotent-cli"
  $lockbox_script = "${script_dir}/lockbox.groovy"
  $groovy_d = "${jenkins_home}/init.groovy.d"

  $docroot = "/var/www/${ci_fqdn}"
  $apache_log_dir = "/var/log/apache2/${ci_fqdn}"


  class { '::jenkins':
    # Preventing the jenkins module from managing the package for us, since
    # we're using the Docker container, see:
    # https://issues.jenkins-ci.org/browse/INFRA-916
    version        => absent,
    repo           => false,
    service_enable => false,
    cli            => false,
  }

  docker::run { 'jenkins':
    image            => 'jenkinsci/jenkins:lts-alpine',
    # This is a "clever" hack to force the init script to pass the numeric UID
    # through on `docker run`. Since passing the string 'jenkins' doesn't
    # actually map the UIDs properly. Using the extra_parameters option because
    # the `username` parameter will get shellescaped in the docker_run_flags()
    # function provided by garethr/docker
    extra_parameters => '-u `id -u jenkins`:`id -g jenkins`',
    # Hard-coding some environment variables because there is no "parent" shell
    # environment to inherit some of these environment settings from.
    # Additionally, Jenkins picks up `user.home` as "?" without the explicit
    # JAVA_OPTS override, breaking the current azure plugin:
    # https://github.com/jenkinsci/azure-slave-plugin/issues/56
    env              => [
      "HOME=${jenkins_home}",
      'USER=jenkins',
      'JAVA_OPTS="-Duser.home=/var/jenkins_home  -Djenkins.install.runSetupWizard=false -Djenkins.model.Jenkins.slaveAgentPort=50000 -Dhudson.model.WorkspaceCleanupThread.retainForDays=2"',
      'JENKINS_OPTS="--httpKeepAliveTimeout=60000"',
    ],
    ports            => ['8080:8080', '50000:50000', '22222:22222'],
    volumes          => ["${jenkins_home}:/var/jenkins_home"],
    pull_on_start    => true,
    require          => [
        File[$jenkins_home],
        User['jenkins'],
    ],
  }

  file { '/etc/default/jenkins':
    ensure  => present,
  }
  # Make sure the old init script is gone, since the package removal won't
  # handle it
  # https://issues.jenkins-ci.org/browse/INFRA-916
  # No-op, just to make puppet-jenkins STFU
  file { '/etc/init.d/jenkins' :
    ensure  => present,
    mode    => '0755',
    content => '#!/bin/sh',
  }


  file { $script_dir:
    ensure => directory,
  }

  # Jenkins custom-bootstrapping
  #
  # These files should be laid down on the file system before Jenkins starts
  # such that they're loaded properly
  ##############################################################################
  file { $groovy_d:
    ensure  => directory,
    owner   => 'jenkins',
    require => [
        User['jenkins'],
        File[$jenkins_home],
    ],
  }

  file { "${groovy_d}/enable-ssh-port.groovy":
    ensure  => present,
    owner   => 'jenkins',
    source  => "puppet:///modules/${module_name}/buildmaster/enable-ssh-port.groovy",
    require => [
        User['jenkins'],
        File[$groovy_d],
    ],
    before  => Docker::Run['jenkins'],
    notify  => Service['docker-jenkins'],
  }

  file { "${groovy_d}/set-up-git.groovy":
    ensure  => present,
    owner   => 'jenkins',
    source  => "puppet:///modules/${module_name}/buildmaster/set-up-git.groovy",
    require => [
        User['jenkins'],
        File[$groovy_d],
    ],
    before  => Docker::Run['jenkins'],
    notify  => Service['docker-jenkins'],
  }

  file { "${groovy_d}/agent-security.groovy":
    ensure  => present,
    owner   => 'jenkins',
    source  => "puppet:///modules/${module_name}/buildmaster/agent-security.groovy",
    require => [
        User['jenkins'],
        File[$groovy_d],
    ],
    before  => Docker::Run['jenkins'],
    notify  => Service['docker-jenkins'],
  }

  file { "${groovy_d}/pipeline-configuration.groovy":
    ensure  => present,
    owner   => 'jenkins',
    source  => "puppet:///modules/${module_name}/buildmaster/pipeline-configuration.groovy",
    require => [
        User['jenkins'],
        File[$groovy_d],
    ],
    before  => Docker::Run['jenkins'],
    notify  => Service['docker-jenkins'],
  }

  file { "${groovy_d}/lock-down-jenkins.groovy":
    ensure  => present,
    require => [
        User['jenkins'],
        File[$groovy_d],
        Exec['generate-cli-ssh-key'],
    ],
    content => template("${module_name}/buildmaster/lockbox.groovy.erb"),
    before  => Docker::Run['jenkins'],
    notify  => Service['docker-jenkins'],
  }

  file { "${groovy_d}/terraform-credentials.groovy":
    ensure  => present,
    require => [
        File[$groovy_d],
        File["${ssh_dir}/azure_k8s.pub"],
    ],
    source  => "puppet:///modules/${module_name}/buildmaster/terraform-credentials.groovy",
    before  => Docker::Run['jenkins'],
    notify  => Service['docker-jenkins'],
  }
  ##############################################################################


  # Prepare Jenkins instance-only SSH keys for CLI usage
  ##############################################################################
  file { $ssh_dir :
    ensure  => directory,
    owner   => 'jenkins',
    mode    => '0700',
    require => [
        User['jenkins'],
        File[$jenkins_home],
    ],
  }
  exec { 'generate-cli-ssh-key':
    require => File[$jenkins_home],
    creates => "${ssh_dir}/${ssh_cli_key}",
    command => "/usr/bin/ssh-keygen -b 4096 -q -f ${ssh_dir}/${ssh_cli_key} -N ''",
  }

  file { "${ssh_dir}/azure_k8s":
    ensure  => absent,
    mode    => '0600',
    content => hiera('azure::k8s::management_ssh_privkey'),
    require => [
        File[$ssh_dir],
    ],
  }

  file { "${ssh_dir}/azure_k8s.pub":
    ensure  => absent,
    mode    => '0644',
    content => hiera('azure::k8s::management_ssh_pubkey'),
    require => [
        File[$ssh_dir],
    ],
  }
  ##############################################################################


  # Bootstrap the Jenkins internal (to Jenkins) user entity for CLI work
  ##############################################################################
  file { "${script_dir}/create-jenkins-cli-user":
    ensure  => present,
    require => File[$script_dir],
    source  => "puppet:///modules/${module_name}/buildmaster/create-jenkins-cli-user",
    mode    => '0755',
  }

  exec { 'create-jenkins-cli-user':
    creates => "${jenkins_home}/users/jenkins/config.xml",
    command => "${script_dir}/create-jenkins-cli-user",
    before  => Docker::Run['jenkins'],
    require => [
      File[$jenkins_home],
      File["${script_dir}/create-jenkins-cli-user"],
    ],
  }
  ##############################################################################

  # CLI support
  ##############################################################################
  file { $cli_script:
    ensure  => present,
    require => File[$script_dir],
    source  => "puppet:///modules/${module_name}/buildmaster/idempotent-cli",
    mode    => '0755',
  }
  exec { 'safe-restart-jenkins-via-ssh-cli':
    require     => [
      File[$cli_script],
    ],
    command     => "${cli_script} safe-restart",
    refreshonly => true,
  }
  ##############################################################################


  file { $lockbox_script :
    ensure  => absent,
  }

  profile::jenkinsplugin { $plugins:
    # Only install plugins after we've secured Jenkins, that seems reasonable
    require => [
      File[$cli_script],
      File["${groovy_d}/lock-down-jenkins.groovy"],
    ],
  }

  exec { 'jenkins-reload-config':
    command     => "${cli_script} reload-configuration",
    refreshonly => true,
    require     => [
      File[$cli_script],
      Exec['generate-cli-ssh-key'],
    ],
  }

  file { [$apache_log_dir, $docroot,]:
    ensure  => directory,
    require => Package['httpd'],
  }

  file { "${docroot}/empty.json" :
    ensure  => file,
    content => '{}',
    mode    => '0644',
    require => File[$docroot],
  }

  file { "${docroot}/empty.xml" :
    ensure  => file,
    content => '<nope/>',
    mode    => '0644',
    require => File[$docroot],
  }

  apache::vhost { $ci_fqdn:
    serveraliases         => [
      # Give all our buildmaster profiles this server alias; it's easier than
      # parameterizing it for compatibility's sake
      'ci.jenkins-ci.org',
    ],
    require               => [
      Docker::Run['jenkins'],
      File[$docroot],
      # We need our installation to be secure before we allow access
      File["${groovy_d}/lock-down-jenkins.groovy"],
    ],
    port                  => 443,
    override              => 'All',
    ssl                   => true,
    docroot               => $docroot,
    error_log_file        => "${ci_fqdn}/error.log",
    access_log_pipe       => "|/usr/bin/rotatelogs -t ${apache_log_dir}/access.log.%Y%m%d%H%M%S 86400",
    proxy_preserve_host   => true,
    allow_encoded_slashes => 'on',
    custom_fragment       => "
RequestHeader set X-Forwarded-Proto \"https\"
RequestHeader set X-Forwarded-Port \"${proxy_port}\"

RewriteEngine on

RewriteCond %{REQUEST_FILENAME} ^(.*)api/xml(.*)$ [NC]
RewriteRule ^.* \"https://jenkins.io/infra/ci-redirects/\"  [L]

# Black hole all traffic to routes like /view/All/people/ which is pretty much
# hit illegitimately used anyways
# See thread dump here: https://gist.github.com/rtyler/f8d02e0c5ff11e03da4e331a0f2ca280
RewriteCond %{REQUEST_FILENAME} ^(.*)people(.*)$ [NC]
RewriteRule ^.* \"https://jenkins.io/infra/ci-redirects/\"  [L]


# Blackhole all the /cli requests over HTTP
RewriteRule ^/cli.* https://github.com/jenkinsci-cert/SECURITY-218

# Loading our Proxy rules ourselves from a custom fragment since the
# puppetlabs/apache module doesn't support ordering of both proxy_pass and
# proxy_pass_match configurations
ProxyRequests Off
ProxyPreserveHost On
ProxyPassMatch (.*)/api/(json|python|xml)(/|$)(.*)  !
ProxyPass / http://localhost:8080/ nocanon
ProxyPassReverse / http://localhost:8080/
",
    aliases               => [
      {
        # Send all api/json or api/python requests to `empty.json` to prevent abusive clients
        # (checkman) from receiving an invalid JSON response and repeatedly attempting
        # to hammer us to get a better response. Works for Python API as well.
        aliasmatch => '(.*)/api/(json|python)(/|$)(.*)',
        path       => "${docroot}/empty.json",
      },
      {
        # Send all api/xml to `empty.xml` to prevent abusive clients (like checkman over
        # in JSON land) from receiving an invalid XML response and repeatedly attempting
        # to hammer us to get a better response.
        aliasmatch => '(.*)/api/xml(/|$)(.*)',
        path       => "${docroot}/empty.xml",
      },
    ],
  }

  apache::vhost { "${ci_fqdn} unsecured":
    serveraliases   => [
      # Give all our buildmaster profiles this server alias; it's easier than
      # parameterizing it for compatibility's sake
      'ci.jenkins-ci.org',
    ],
    servername      => $ci_fqdn,
    port            => 80,
    docroot         => $docroot,
    redirect_status => 'permanent',
    redirect_dest   => "https://${ci_fqdn}/",
    error_log_file  => "${ci_fqdn}/error_nonssl.log",
    access_log_pipe => '/dev/null',
    require         => Apache::Vhost[$ci_fqdn],
  }


  # This is a legacy role imported from infra-puppet, thus the goofy numbering
  firewall { '108 Jenkins CLI port' :
    proto  => 'tcp',
    port   => 47278,
    action => 'accept',
  }

  firewall { '801 Allow Jenkins web access only on localhost':
    proto   => 'tcp',
    port    => 8080,
    action  => 'accept',
    iniface => 'lo',
  }

  firewall { '802 Block external Jenkins web access':
    proto  => 'tcp',
    port   => 8080,
    action => 'drop',
  }

  firewall { '803 Expose JNLP port':
    proto  => 'tcp',
    port   => 50000,
    action => 'accept',
  }

  firewall { '810 Jenkins CLI SSH':
    proto  => 'tcp',
    port   => 22222,
    action => 'accept',
  }

  # We can only acquire certs in production due to the way the letsencrypt
  # challenge process works
  if (($letsencrypt == true) and ($::environment == 'production') and ($::vagrant != '1')) {
    letsencrypt::certonly { $ci_fqdn:
      domains     => [$ci_fqdn],
      plugin      => 'apache',
      manage_cron => true,
    }

    Apache::Vhost <| title == $ci_fqdn |> {
      ssl_key       => "/etc/letsencrypt/live/${ci_fqdn}/privkey.pem",
      # When Apache is upgraded to >= 2.4.8 this should be changed to
      # fullchain.pem
      ssl_cert      => "/etc/letsencrypt/live/${ci_fqdn}/cert.pem",
      ssl_chain     => "/etc/letsencrypt/live/${ci_fqdn}/chain.pem",
    }
  }
}
